#include "rsrtm.h"
#include "rsrtm_read_waveform_example.h"
#include <utility.h>  


#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else

/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViSession  instrSession = VI_NULL;
static 	 int 	 	AcquisitionID;
static	 int		stop = 0; /*control measurement*/

static	 ViReal64*  waveForm = NULL; 
static	 ViInt32    length;

/* Global functions */
int CVICALLBACK startAcquisition (void *functionData);

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsrtm_read_waveform_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViChar      instr_modl [STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     arithmetics      = 0;
    ViInt32     acqType          = 0;
    ViStatus    status           = VI_SUCCESS;
    ViReal64    acqTime          = 0.0;
    ViReal64    vertScale        = 0.0;
    ViReal64    offset           = 0.0;
	ViReal64    horizScale       = 0.0;
	ViReal64    triggerLevel     = 0.0;
	ViInt32    	triggerSlope	 = 0;
	ViReal64    triggerDelay     = 0.0;

    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read values from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);
                GetCtrlVal (panelHandle, PANEL_ARITHMETICS, &arithmetics);
                GetCtrlVal (panelHandle, PANEL_ACQTYPE, &acqType);
                GetCtrlVal (panelHandle, PANEL_VERTSCALE, &vertScale);
                GetCtrlVal (panelHandle, PANEL_OFFSET, &offset);
				GetCtrlVal (panelHandle, PANEL_HORIZSCALE, &horizScale);
				GetCtrlVal (panelHandle, PANEL_TRIG_LEVEL, &triggerLevel);
				GetCtrlVal (panelHandle, PANEL_TRIG_SLOPE, &triggerSlope);


                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                CHECKERR (rsrtm_init (instr_desc, id, reset, &instrSession));
                
                /* configures how the oscilloscope acquires data and fills the waveform record */
                CHECKERR (rsrtm_ConfigureAcquisitionType (instrSession ,RSRTM_VAL_CHANNEL_1, acqType));
                
                /* configures the method to reduce the data stream of the ADC to 
                   a stream of waveform points with lower sample rate */
                CHECKERR (rsrtm_ConfigureWaveformArithmetic (instrSession, RSRTM_VAL_CHANNEL_1, arithmetics));
                
                /* configures the time of one acquisition, that is the time across
                   the 10 divisions of the diagram */
                CHECKERR (rsrtm_ConfigureAcquisitionRecord (instrSession, 10*horizScale, 0));
			
				/* configures the axis scaling */
                CHECKERR (rsrtm_ConfigureChannelVerticalScale(instrSession, RSRTM_VAL_CHANNEL_1, vertScale));
				CHECKERR (rsrtm_ConfigureTimeBase (instrSession, horizScale, 50));
				
				/* trigger configuration */
				CHECKERR (rsrtm_ConfigureEdgeTriggerSource (instrSession, RSRTM_VAL_TRIGGER_A, RSRTM_VAL_CHANNEL_1, 1, triggerLevel, triggerSlope));
				
                /* configures the offset voltage, the offset is subtracted to
                   correct an offset-affected signal */
                CHECKERR (rsrtm_ConfigureChannelOffset(instrSession, RSRTM_VAL_CHANNEL_1, offset));
                 
                
                SetWaitCursor (0);
				/* start waveform acquisition */
                CmtScheduleThreadPoolFunction (DEFAULT_THREAD_POOL_HANDLE, startAcquisition, NULL, &AcquisitionID);
            }
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: startAcquisition CallBack
/* Purpose:  Initializes continual measurement, acquire and dislpay data
/*===========================================================================*/
int CVICALLBACK startAcquisition (void *functionData)
{
	ViInt32     actualPoints     = 0;
    ViReal64    initialX          = 0.0;
    ViReal64    xIncrement       = 0.0;
	ViStatus    status           = VI_SUCCESS;
	ViReal64    timeout          = 0.0;
	
	stop = 0; //reset global var
	
	GetCtrlVal (panelHandle, PANEL_TIMEOUT, &timeout);
	
	/* get waveform points count */
	CHECKERR (rsrtm_QueryWaveformSamplesRange (instrSession, RSRTM_VAL_CHANNEL_1, &length));
	
	if (waveForm)														 
    {
        free (waveForm);
        waveForm = NULL;
    }
    waveForm = (ViReal64*) malloc (sizeof (ViReal64) * length);
	
	if (waveForm == NULL)
	{
		MessagePopup ("Error", "Unable allocate memory for results");
		return 0;
	}
		
	/* initiates a waveform acquisition */
	CHECKERR (rsrtm_InitiateAcquisition(instrSession, RSRTM_VAL_INIT_SINGLE));

	while (!stop)
	{
		/* This function initiates an acquisition on the channels that you
		   enable with the rsrtm_ConfigureChannel function.  It then
		   waits for the acquisition to complete, and returns the waveform
		   for the channel you specify*/ 

		CHECKERR (rsrtm_ReadWaveform (instrSession, RSRTM_VAL_CHANNEL_1, length, timeout, 
		    waveForm, &actualPoints, &initialX, &xIncrement));

		/* Deletes one or all plots from a graph control */
		CHECKERR (DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW));

		/* Plots a waveform onto a graph control */
		CHECKERR (PlotWaveform (panelHandle, PANEL_GRAPH, waveForm, actualPoints, VAL_DOUBLE,
		            1.0, 0.0, initialX, 1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
		            VAL_SOLID, 1, VAL_YELLOW));
	}
	
	return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsrtm_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsrtm_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_START, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_ARITHMETICS, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_ACQTYPE, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_VERTSCALE, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_OFFSET, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_HORIZSCALE, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_TRIG_LEVEL, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_TRIG_SLOPE, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_TIMEOUT, ATTR_DIMMED, !enable);
	SetCtrlAttribute (panelHandle, PANEL_EXIT, ATTR_DIMMED, !enable);
	
	SetCtrlAttribute (panelHandle, PANEL_STOP, ATTR_DIMMED, enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
	if (instrSession)
	{
	    if (deviceClear)
	    {
	        viClear (instrSession);
	        viPrintf (instrSession, "*CLS\n");
	    }
    
	    /* close instrument session */
	    rsrtm_close (instrSession);
	    instrSession = VI_NULL;
	}
    
    /* enable GUI */
    EnableButtons (1);
    
    /* enable mouse events*/
    SetWaitCursor (0);
    
    if (waveForm)
    {
        free (waveForm);
        waveForm = NULL;
    }
}


/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stop measurement, close instrument session and re-enable GUI.
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			stop = 1;
			CmtWaitForThreadPoolFunctionCompletion (DEFAULT_THREAD_POOL_HANDLE, AcquisitionID, OPT_TP_PROCESS_EVENTS_WHILE_WAITING);
			CmtReleaseThreadPoolFunctionID (DEFAULT_THREAD_POOL_HANDLE, AcquisitionID);
			
			SetCtrlVal (panelHandle, PANEL_START, 0);
			SetCtrlVal (panelHandle, PANEL_STOP, 0);
			
			/* cleanup and close driver session */
			cleanup (VI_TRUE);
			break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: Exit Button Callback
/* Purpose:  Terminates the example application.
/*===========================================================================*/
int CVICALLBACK exitButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
	}
	return 0;
}
